﻿using System;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Web;
using log4net;
using log4net.Config;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WmsServerEdition;

namespace WmsPlugin
{
    public class LogServerRequestsWmsLayerPlugin : WmsLayerPlugin
    {
        private static readonly ILog log = log4net.LogManager.GetLogger("FileLogger");

        public LogServerRequestsWmsLayerPlugin()
        {
            XmlConfigurator.ConfigureAndWatch(new FileInfo(Directory.GetParent(Directory.GetParent(AppDomain.CurrentDomain.BaseDirectory).FullName).FullName + "\\SamplePlugin\\LogServerRequestsWmsLayerPluginLogging.config"));
        }

        // This method is only called once per style and crs.  In it you should create your
        // layers and add them to the MapConfiguration.  If you want to use tile caching you
        // can also specif that in the MapConfiguration under the TileCache property.
        // If you have setup multiple styles or projections this method will get called for
        // each unique combination
        protected override MapConfiguration GetMapConfigurationCore(string style, string crs)
        {
            WorldMapKitLayer worldMapKitLayer = new WorldMapKitLayer();

            // Get the directory to the sample data
            string worldLayerFilePath = Directory.GetParent(Directory.GetParent(AppDomain.CurrentDomain.BaseDirectory).FullName).FullName + "\\SampleData\\Countries02.shp";

            // Create the world layer from a shapefile and add a style to it
            ShapeFileFeatureLayer worldLayer = new ShapeFileFeatureLayer(worldLayerFilePath);
            worldLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Country1;
            worldLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            // Create a MapConfiguration and add the layer to it
            MapConfiguration mapConfiguration = new MapConfiguration();
            mapConfiguration.Layers.Add("WorldMapKitLayer", worldMapKitLayer);
            mapConfiguration.Layers.Add("WorldLayer", worldLayer);

            return mapConfiguration;
        }

        // In this method you need to return the name of the Layer that WMS will expose.
        // You will use this name on the client to specify the layer you want to consume
        protected override string GetNameCore()
        {
            return "Log Server Requests";
        }

        // In this method you need to return the projections that are supported by your data.
        // It is your responsability to project the data in the MapConfiguration for each projection
        // type you specify here.
        protected override Collection<string> GetProjectionsCore()
        {
            return new Collection<string> { "EPSG:4326" };
        }

        // In this method you need to return the bounding box of the layer.
        protected override RectangleShape GetBoundingBoxCore(string crs)
        {
            return new RectangleShape(-126.826171875, 57.104766845702, -70.83984375, 18.960235595702);
        }

        protected override Bitmap GetMapCore(GetMapRequest getMapRequest, MapConfiguration cachedMapConfiguration, HttpContext context)
        {
            log.Debug("Start GetMapCore method of LogServerRequestsWmsLayerPlugin");

            Stopwatch stopwatch = new Stopwatch();
            stopwatch.Start();
            Bitmap bitmap = base.GetMapCore(getMapRequest, cachedMapConfiguration, context);
            stopwatch.Stop();

            log.Debug("ElapsedMilliseconds:" + stopwatch.ElapsedMilliseconds + "ms");
            return bitmap;
        }
    }
}
